package com.diagbot.process;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.OtherTipTransfusionNeoDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LibTypeEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.RuleVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 其他提示总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class OtherTipProcess {

    @Autowired
    GroupRule groupRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;

    /**
     * 处理业务——化验
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.other.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherLis(lis, ruleBaseDTOList, otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 处理业务——辅检
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processPacs(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<Item> itemList = pacsLabel.getRes();
        List<BillMsg> otherList = res.getOtherList();
        for (Item item : itemList) {
            RuleVO ruleVO = getRuleVO(item);
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    pacsRule.otherPacs(ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 处理业务——输血
     *
     * @param wordCrfDTO
     * @param res
     */
    public void processTransfusion(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        List<Lis> lisList = wordCrfDTO.getLis();
        List<BillMsg> otherList = res.getOtherList();
        for (Lis lis : lisList) {
            RuleVO ruleVO = getRuleVO(lis, RuleTypeEnum.transfusion.getKey());
            // 1、获取相关数据
            if (ruleVO != null) {
                RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
                if (ruleExtDTO == null) {
                    continue;
                }
                List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
                for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                    List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                    // 2、规则匹配
                    lisRule.otherTransfusion(lis, ruleBaseDTOList, ruleConditionDTO.getMsg(), otherList, wordCrfDTO);
                }
            }
        }
    }

    /**
     * 获取其他值提醒入参——辅检
     *
     * @param bean
     * @return
     */
    public RuleVO getRuleVO(Item bean) {
        if (StringUtil.isNotBlank(bean.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(bean.getName());
            ruleVO.setLibName(bean.getUniqueName());
            ruleVO.setLibType(LibTypeEnum.pacsResult.getKey());
            ruleVO.setLibTypeName(TypeEnum.pacs.getName());
            ruleVO.setDateValue(bean.getDateValue());
            ruleVO.setSuffix(String.valueOf(RuleTypeEnum.other.getKey()));
            return ruleVO;
        }
        return null;
    }

    /**
     * 获取其他值提醒入参——化验
     *
     * @param lis
     * @return
     */
    public RuleVO getRuleVO(Lis lis, Integer key) {
        if (StringUtil.isNotBlank(lis.getUniqueName())) {
            RuleVO ruleVO = new RuleVO();
            ruleVO.setInputName(lis.getName());
            ruleVO.setLibName(lis.getUniqueName());
            ruleVO.setLibType(LibTypeEnum.lisDetail.getKey());
            ruleVO.setLibTypeName(TypeEnum.lis.getName());
            ruleVO.setDateValue(lis.getDateValue());
            ruleVO.setSuffix(String.valueOf(key));
            return ruleVO;
        }
        return null;
    }

    /**
     * 处理业务——输血
     *
     * @param otherTipTransfusionNeoDTOList
     * @param res
     */
    public void processTransfusion(List<OtherTipTransfusionNeoDTO> otherTipTransfusionNeoDTOList, IndicationDTO res) {
        for (OtherTipTransfusionNeoDTO bean : otherTipTransfusionNeoDTOList) {
            if (StringUtil.isBlank(bean.getName())) {
                continue;
            }
            StringBuffer sb = new StringBuffer();
            // 诊断
            if (ListUtil.isNotEmpty(bean.getDiag())) {
                sb.append("诊断");
                List<String> diag = bean.getDiag().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(diag, ","));
            }
            // 辅检结果
            if (ListUtil.isNotEmpty(bean.getPacsRes())) {
                List<String> pacsRes = bean.getPacsRes().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(pacsRes, ","));
            }
            // 化验
            if (ListUtil.isNotEmpty(bean.getLisList())) {
                for (Lis lis : bean.getLisList()) {
                    if (sb.toString().length() > 0) {
                        sb.append("，");
                    }
                    if (lis.getName() == null || lis.getDetailName() == null
                            || !lis.getName().equals(lis.getDetailName())) {
                        sb.append(lis.getName() + lis.getDetailName() + CoreUtil.getLisValue(lis));
                    } else {
                        sb.append(lis.getDetailName() + CoreUtil.getLisValue(lis));
                    }
                }
            }
            // 辅检名称
            if (ListUtil.isNotEmpty(bean.getPacs())) {
                if (sb.toString().length() > 0) {
                    sb.append("，");
                }
                sb.append("已开");
                List<String> pacs = bean.getPacs().stream().map(r -> r.getName()).collect(Collectors.toList());
                sb.append(StringUtils.join(pacs, ","));
            }
            // 手术
            if (ListUtil.isNotEmpty(bean.getOperation())) {
                List<String> operation = bean.getOperation().stream().map(r -> r.getName()).collect(Collectors.toList());
                if (sb.toString().length() > 0) {
                    sb.append("，");
                }
                sb.append("已开");
                sb.append(StringUtils.join(operation, ","));
            }
            BillMsg billMsg = MsgUtil.getCommonOtherTipTransfusionMsg(sb.toString(), bean.getName());
            res.getOtherList().add(billMsg);
        }
    }
}
